@@ -2,6 +2,8 @@ import doctest
 import math
 import os
 import unittest
+import warnings
+from contextlib import contextmanager
 
 from pint import UnitRegistry
 from pint.testsuite.helpers import PintOutputChecker
@@ -23,6 +25,14 @@ class QuantityTestCase:
         cls.U_ = None
 
 
+@contextmanager
+def assert_no_warnings():
+    with warnings.catch_warnings():
+        warnings.simplefilter("error")
+
+        yield
+
+
 def testsuite():
     """A testsuite that has all the pint tests."""
     suite = unittest.TestLoader().discover(os.path.dirname(__file__))
