@@ -20,7 +20,7 @@ from ._typing import UnitLike
 from .compat import NUMERIC_TYPES, babel_parse, is_upcast_type
 from .definitions import UnitDefinition
 from .errors import DimensionalityError
-from .formatting import extract_custom_flags, format_unit
+from .formatting import extract_custom_flags, format_unit, split_format
 from .util import PrettyIPython, SharedRegistryObject, UnitsContainer
 
 if TYPE_CHECKING:
@@ -80,8 +80,10 @@ class Unit(PrettyIPython, SharedRegistryObject):
         return "<Unit('{}')>".format(self._units)
 
     def __format__(self, spec) -> str:
-        spec = spec or extract_custom_flags(self.default_format)
-        if "~" in spec:
+        _, uspec = split_format(
+            spec, self.default_format, self._REGISTRY.separate_format_defaults
+        )
+        if "~" in uspec:
             if not self._units:
                 return ""
             units = UnitsContainer(
@@ -90,11 +92,11 @@ class Unit(PrettyIPython, SharedRegistryObject):
                     for key, value in self._units.items()
                 )
             )
-            spec = spec.replace("~", "")
+            uspec = uspec.replace("~", "")
         else:
             units = self._units
 
-        return format_unit(units, spec, registry=self._REGISTRY)
+        return format_unit(units, uspec, registry=self._REGISTRY)
 
     def format_babel(self, spec="", locale=None, **kwspec: Any) -> str:
         spec = spec or extract_custom_flags(self.default_format)
@@ -168,14 +170,16 @@ class Unit(PrettyIPython, SharedRegistryObject):
         -------
         bool
         """
-        if contexts:
+        from .quantity import Quantity
+
+        if contexts or self._REGISTRY._active_ctx:
             try:
                 (1 * self).to(other, *contexts, **ctx_kwargs)
                 return True
             except DimensionalityError:
                 return False
 
-        if isinstance(other, (self._REGISTRY.Quantity, self._REGISTRY.Unit)):
+        if isinstance(other, (Quantity, Unit)):
             return self.dimensionality == other.dimensionality
 
         if isinstance(other, str):
@@ -225,7 +229,7 @@ class Unit(PrettyIPython, SharedRegistryObject):
 
     def __pow__(self, other) -> "Unit":
         if isinstance(other, NUMERIC_TYPES):
-            return self.__class__(self._units ** other)
+            return self.__class__(self._units**other)
 
         else:
             mess = "Cannot power Unit by {}".format(type(other))
