@@ -423,10 +423,11 @@ class UnitsContainer(Mapping):
 
     # Only needed by pickle protocol 0 and 1 (used by pytables)
     def __getstate__(self):
-        return self._d, self._hash, self._one, self._non_int_type
+        return self._d, self._one, self._non_int_type
 
     def __setstate__(self, state):
-        self._d, self._hash, self._one, self._non_int_type = state
+        self._d, self._one, self._non_int_type = state
+        self._hash = None
 
     def __eq__(self, other) -> bool:
         if isinstance(other, UnitsContainer):
@@ -519,7 +520,7 @@ class UnitsContainer(Mapping):
             err = "Cannot divide {} by UnitsContainer"
             raise TypeError(err.format(type(other)))
 
-        return self ** -1
+        return self**-1
 
 
 class ParserHelper(UnitsContainer):
@@ -719,7 +720,7 @@ class ParserHelper(UnitsContainer):
         d = self._d.copy()
         for key in self._d:
             d[key] *= other
-        return self.__class__(self.scale ** other, d, non_int_type=self._non_int_type)
+        return self.__class__(self.scale**other, d, non_int_type=self._non_int_type)
 
     def __truediv__(self, other):
         if isinstance(other, str):
@@ -906,29 +907,50 @@ def to_units_container(
             return UnitsContainer(unit_like)
 
 
-def infer_base_unit(q):
+def infer_base_unit(
+    unit_like: Union[UnitLike, Quantity], registry: Optional[BaseRegistry] = None
+) -> UnitsContainer:
     """
+    Given a Quantity or UnitLike, give the UnitsContainer for it's base units.
 
     Parameters
     ----------
-    q :
+    unit_like : Union[UnitLike, Quantity]
+        Quantity or Unit to infer the base units from.
 
+    registry: Optional[BaseRegistry]
+        If provided, uses the registry's UnitsContainer and parse_unit_name.  If None,
+        uses the registry attached to unit_like.
 
     Returns
     -------
-    type
+    UnitsContainer
 
+    Raises
+    ------
+    ValueError
+        The unit_like did not reference a registry, and no registry was provided.
 
     """
     d = udict()
-    for unit_name, power in q._units.items():
-        candidates = q._REGISTRY.parse_unit_name(unit_name)
+
+    original_units = to_units_container(unit_like, registry)
+
+    if registry is None and hasattr(unit_like, "_REGISTRY"):
+        registry = unit_like._REGISTRY
+    if registry is None:
+        raise ValueError("No registry provided.")
+
+    for unit_name, power in original_units.items():
+        candidates = registry.parse_unit_name(unit_name)
         assert len(candidates) == 1
         _, base_unit, _ = candidates[0]
         d[base_unit] += power
 
     # remove values that resulted in a power of 0
-    return UnitsContainer({k: v for k, v in d.items() if v != 0})
+    nonzero_dict = {k: v for k, v in d.items() if v != 0}
+
+    return registry.UnitsContainer(nonzero_dict)
 
 
 def getattr_maybe_raise(self, item):
@@ -971,7 +993,7 @@ class SourceIterator:
 
     """
 
-    def __new__(cls, sequence):
+    def __new__(cls, sequence, filename=None, is_resource=False):
         if isinstance(sequence, SourceIterator):
             return sequence
 
@@ -980,6 +1002,8 @@ class SourceIterator:
         if sequence is not None:
             obj.internal = enumerate(sequence, 1)
             obj.last = (None, None)
+            obj.filename = filename or getattr(sequence, "name", None)
+            obj.is_resource = is_resource
 
         return obj
 
